==> 2.4.6.tar.gz: android/src/main/kotlin/com/fireworksdk/bridge/flutter/handler/FWShoppingModuleHandler.kt <==
package com.fireworksdk.bridge.flutter.handler

import android.content.Context
import android.widget.Toast
import com.firework.common.product.CurrencyCode
import com.firework.common.product.Product
import com.firework.error.shopping.ShoppingError
import com.firework.sdk.FireworkSdk
import com.firework.shopping.LinkButtonOptions
import com.firework.shopping.ProductCardsOptions
import com.firework.shopping.ProductDetailsOptions
import com.firework.shopping.ProductHydrator
import com.firework.shopping.Shopping
import com.firework.shopping.ShoppingCtaButtonOptions
import com.firework.shopping.ShoppingTheme
import com.firework.shopping.ShoppingViewOptions
import com.fireworksdk.bridge.flutter.models.FWShoppingModuleCallMethod
import com.fireworksdk.bridge.flutter.utils.FWEventUtils
import com.fireworksdk.bridge.models.FWProductInfoViewConfigurationDeserializer
import com.fireworksdk.bridge.models.FWShoppingCtaResultDeserializer
import com.fireworksdk.bridge.models.FWVideoShoppingProduct
import com.fireworksdk.bridge.models.FWVideoShoppingProductDeserializer
import com.fireworksdk.bridge.models.enums.FWCtaButtonTextValue
import com.fireworksdk.bridge.models.enums.FWProductCardCtaButtonTextValue
import com.fireworksdk.bridge.models.enums.FWProductCardTheme
import com.fireworksdk.bridge.models.enums.FWShoppingCtaResultRes
import com.fireworksdk.bridge.models.weakProperty
import com.fireworksdk.bridge.utils.FWCommonUtil
import com.fireworksdk.bridge.utils.FWGlobalDataUtil
import com.fireworksdk.bridge.utils.FWLogUtils
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.common.MethodChannel.Result
import org.json.JSONArray
import org.json.JSONObject


class FWShoppingModuleHandler(
  private val context: Context,
  channel: MethodChannel?
) : MethodChannel.MethodCallHandler {

  private val methodChannel by weakProperty<MethodChannel?>(channel)
  private val ctaClickChannelResultSet = mutableSetOf<String>()

  override fun onMethodCall(call: MethodCall, result: Result) {
    when (call.method) {
      FWShoppingModuleCallMethod.Init.rawValue -> {
        cartListener()
        productListener()
        result.success(null)
      }

      FWShoppingModuleCallMethod.SetCartIconVisible.rawValue -> {
        val value: Boolean? = call.arguments as Boolean?
        if (value == true) {
          FireworkSdk.shopping.setShoppingCartBehaviour(Shopping.CartBehaviour.Callback)
        } else {
          FireworkSdk.shopping.setShoppingCartBehaviour(Shopping.CartBehaviour.NoCart)
        }
        result.success(null)
      }

      FWShoppingModuleCallMethod.SetCartItemCount.rawValue -> {
        val value: Int? = call.arguments as Int?
        FireworkSdk.shopping.numberOfItemsInCart = value ?: 0
        result.success(null)
      }

      FWShoppingModuleCallMethod.SetCustomClickLinkButtonEnabled.rawValue -> {
        val value: Boolean? = call.arguments as Boolean?
        FWGlobalDataUtil.customClickLinkButtonEnabled = value?:false
        result.success(true)
      }

      FWShoppingModuleCallMethod.SetProductInfoViewConfiguration.rawValue -> {
        val configMap = call.arguments
        val jsonObject = JSONObject(configMap as HashMap<*, *>)
        val configModel = FWProductInfoViewConfigurationDeserializer.deserialize(jsonObject)
        FWLogUtils.d { "FWVideoShoppingModule setProductInfoViewConfiguration: $configModel" }
        if (configModel != null) {
          val isLinkButtonHidden = configModel.linkButton?.isHidden == true
          var text = ShoppingCtaButtonOptions.Text.ADD_TO_CART
          when (configModel.ctaButton?.text) {
            FWCtaButtonTextValue.ShopNow -> {
              text = ShoppingCtaButtonOptions.Text.SHOP_NOW
            }
            else -> {}
          }

          var productCtaButtonText = ProductCardsOptions.Text.SHOP_NOW
          when (configModel.productCard?.ctaButtonText) {
            FWProductCardCtaButtonTextValue.BuyNow -> {
              productCtaButtonText = ProductCardsOptions.Text.BUY_NOW
            }
            else -> {}
          }

          var productCardTheme = ShoppingTheme.DARK
          when (configModel.productCard?.theme) {
            FWProductCardTheme.Light -> {
              productCardTheme = ShoppingTheme.LIGHT
            }
            else -> {}
          }

          var productCardCornerRadius = FWCommonUtil.dpToPx(4.toFloat(), context)
          if (configModel.productCard?.cornerRadius != null) {
            productCardCornerRadius = FWCommonUtil.dpToPx(configModel.productCard.cornerRadius.toFloat(), context)
          }

          var isPriceVisible = true
          if (configModel.productCard?.priceConfiguration?.isHidden != null) {
            isPriceVisible = !configModel.productCard.priceConfiguration.isHidden
          }

          var isCtaVisible = true
          if (configModel.productCard?.isCtaButtonHidden != null) {
            isCtaVisible = !configModel.productCard.isCtaButtonHidden
          }

          FireworkSdk.shopping.setShoppingViewOptions(
            ShoppingViewOptions(
              productCardsOptions = ProductCardsOptions(
                ctaButtonText = productCtaButtonText,
                cornerRadius = productCardCornerRadius.toFloat(),
                isPriceVisible = isPriceVisible,
                isCtaVisible = isCtaVisible,
              ),
              productDetailsOptions = ProductDetailsOptions(
                linkButtonOptions = LinkButtonOptions(isVisible = !isLinkButtonHidden),
                shoppingCtaButtonOptions = ShoppingCtaButtonOptions(text = text),
              ),
              theme = productCardTheme
            ),
          )
        }

        result.success(true)
      }

      FWShoppingModuleCallMethod.SetCustomTapProductCardEnabled.rawValue -> {
        val value: Boolean? = call.arguments as Boolean?
        if (value == true) {
          FireworkSdk.shopping.setOnProductCardClickListener { productId, unitId, productWebUrl ->
            FWLogUtils.d { "FWVideoShoppingModule productCardClick $productId $unitId $productWebUrl" }

            FWEventUtils.sendProductCardClickEvent(
              methodChannel,
              productWebUrl,
              productId,
              unitId
            )
          }
        } else {
          FireworkSdk.shopping.setOnProductCardClickListener(null)
        }
        result.success(true)
      }

      FWShoppingModuleCallMethod.ShowShoppingCTALoader.rawValue -> {
        val configMap = call.arguments as HashMap<*, *>
        val callbackId = configMap["callbackId"] as String?
        if (callbackId.isNullOrBlank()) {
          result.success(false)
          return
        }
        if (!ctaClickChannelResultSet.contains(callbackId)) {
          result.success(false)
          return
        }

        FireworkSdk.shopping.setCtaButtonStatus(Shopping.CtaButtonStatus.Loading)
        result.success(true)
      }

      FWShoppingModuleCallMethod.CompleteShoppingCTAHandlingProcess.rawValue -> {
        val configMap = call.arguments as HashMap<*, *>
        val callbackId = configMap["callbackId"] as String?
        if (callbackId.isNullOrBlank()) {
          result.success(false)
          return
        }

        if (!ctaClickChannelResultSet.contains(callbackId)) {
          result.success(false)
          return
        }
        ctaClickChannelResultSet.remove(callbackId)

        val resultMap = configMap["result"] as HashMap<*, *>
        val jsonObject = JSONObject(resultMap)
        val resultModel = FWShoppingCtaResultDeserializer.deserialize(jsonObject)

        val res = resultModel?.res
        val tips = resultModel?.tips
        res?.let {
          when (it) {
            FWShoppingCtaResultRes.Success -> {
              val numberOfItemsInCart = FireworkSdk.shopping.numberOfItemsInCart
              FireworkSdk.shopping.setCtaButtonStatus(Shopping.CtaButtonStatus.Success)
              FireworkSdk.shopping.numberOfItemsInCart = numberOfItemsInCart
            }

            FWShoppingCtaResultRes.Fail -> {
              FireworkSdk.shopping.setCtaButtonStatus(Shopping.CtaButtonStatus.Error)
            }

            FWShoppingCtaResultRes.Loading -> {
              FireworkSdk.shopping.setCtaButtonStatus(Shopping.CtaButtonStatus.Loading)
            }

            else -> {
            }
          }
        }

        if (!tips.isNullOrBlank()) {
          Toast.makeText(context, tips, Toast.LENGTH_LONG).show()
        }

        result.success(true)
      }

      else -> {
        result.notImplemented()
      }
    }
  }

  private fun cartListener() {
    FireworkSdk.shopping.setOnCartClickListener {
      FWLogUtils.d { "FWVideoShoppingModule cartClicked" }
      FWEventUtils.sendClickCartIconEvent(methodChannel)
    }

    FireworkSdk.shopping.setOnCtaButtonClicked { productId, unitId, productWebUrl ->
      FWLogUtils.d { "FWVideoShoppingModule updateCart, productId: $productId, unitId: $unitId" }
      val channelResult = object : Result {
        override fun success(result: Any?) {
        }

        override fun error(
          errorCode: String,
          errorMessage: String?,
          errorDetails: Any?
        ) {
        }

        override fun notImplemented() {
        }
      }

      val callbackId = channelResult.hashCode().toString()
      ctaClickChannelResultSet.add(callbackId)
      FWEventUtils.sendShoppingCtaButtonClickEvent(methodChannel, productId, unitId, productWebUrl, callbackId, channelResult)
    }

    FireworkSdk.shopping.setOnShoppingErrorListener(
      object : Shopping.OnShoppingErrorListener {
        override fun onShoppingError(error: ShoppingError) {
          when (error) {
            is ShoppingError.CtaButtonClickError.Timeout -> {
            }
            else -> {}
          }
        }
      },
    )
  }

  private fun productListener() {
    FireworkSdk.shopping.setOnProductHydrationListener(object : Shopping.OnProductHydrationListener {
      override fun onProductHydration(products: List<Product>, hydrator: ProductHydrator) {
        FWLogUtils.d { "FWVideoShoppingModule hydrateProducts, hydrator: ${hydrator.hashCode()}" }
        if (products.isEmpty()) {
          return
        }
        FWEventUtils.sendUpdateProductsDetailsEvent(methodChannel, products, object : Result {
          override fun success(result: Any?) {
            FWLogUtils.d { "FWVideoShoppingModule updateVideoProduct: $result" }
            if (result is ArrayList<*>) {
              val jsonArray = JSONArray(result)
              val videoShoppingProducts =
                FWVideoShoppingProductDeserializer.deserializeArray(jsonArray)
              if (videoShoppingProducts.isNullOrEmpty()) {
                return
              }
              videoShoppingProducts.forEachIndexed { i, vp ->
                val p = products.find { p ->
                  p.id == vp.productId
                }
                p?.let {
                  hydrateProduct(it, vp, hydrator, i)
                }
              }
              hydrator.completeHydration()
            }
          }

          override fun error(
            errorCode: String,
            errorMessage: String?,
            errorDetails: Any?
          ) {
          }

          override fun notImplemented() {
          }
        })

      }
    })

    FireworkSdk.shopping.setOnProductLinkClickListener { productId, unitId, productWebUrl ->
      if (FWGlobalDataUtil.customClickLinkButtonEnabled) {
        FWEventUtils.sendCustomLinkButtonClickEvent(
          methodChannel,
          productWebUrl,
          productId,
          unitId
        )
      }
      FWGlobalDataUtil.customClickLinkButtonEnabled
    }
  }

  private fun hydrateProduct(
    product: Product,
    videoProduct: FWVideoShoppingProduct,
    hydrator: ProductHydrator,
    position: Int
  ) {
    FWLogUtils.d { "FWVideoShoppingModule hydrateProduct: $position" }
    product.id?.let { id ->
      val name = videoProduct.name ?: product.name
      val description = (videoProduct.description ?: product.description) ?: ""

      hydrator.hydrate(id) {
        name(name)
        description(description)
        product.units.forEachIndexed { _, unit ->
          unit.id?.let { variantId ->
            val vpUnit = videoProduct.units?.find { vpu ->
              vpu.unitId == variantId
            }
            vpUnit?.let { vpu ->
              val currency = if (vpu.price?.currencyCode != null && vpu.price.currencyCode.isNotBlank())
                CurrencyCode.valueOf(vpu.price.currencyCode)
              else unit.price.currencyCode

              variant(variantId) {
                imageUrl((vpu.imageUrl ?: unit.image?.url) ?: "")
                url(vpu.url ?: unit.url)
                currency(currency)
                price(price = vpu.price?.amount ?: unit.price.amount)
                name((vpu.name ?: unit.name) ?: "")
                isAvailable(vpu.isAvailable) // i != 0
              }
            }
          }
        }
        isAvailable(videoProduct.isAvailable) // position != 0
      }
    }
  }

}

==> 2.8.0.tar.gz: android/src/main/kotlin/com/fireworksdk/bridge/flutter/handler/FWShoppingModuleHandler.kt <==
package com.fireworksdk.bridge.flutter.handler

import android.content.Context
import android.widget.Toast
import com.firework.analyticsevents.VideoInfo
import com.firework.common.product.CurrencyCode
import com.firework.common.product.Product
import com.firework.error.shopping.ShoppingError
import com.firework.sdk.FireworkSdk
import com.firework.shopping.LinkButtonOptions
import com.firework.shopping.ProductCardsOptions
import com.firework.shopping.ProductDetailsOptions
import com.firework.shopping.ProductHydrator
import com.firework.shopping.Shopping
import com.firework.shopping.ShoppingCtaButtonOptions
import com.firework.shopping.ShoppingTheme
import com.firework.shopping.ShoppingViewOptions
import com.fireworksdk.bridge.flutter.models.FWShoppingModuleCallMethod
import com.fireworksdk.bridge.flutter.utils.FWEventUtils
import com.fireworksdk.bridge.models.FWProductInfoViewConfigurationDeserializer
import com.fireworksdk.bridge.models.FWShoppingCtaResultDeserializer
import com.fireworksdk.bridge.models.FWVideoShoppingProduct
import com.fireworksdk.bridge.models.FWVideoShoppingProductDeserializer
import com.fireworksdk.bridge.models.enums.FWCtaButtonTextValue
import com.fireworksdk.bridge.models.enums.FWProductCardCtaButtonTextValue
import com.fireworksdk.bridge.models.enums.FWProductCardTheme
import com.fireworksdk.bridge.models.enums.FWShoppingCtaResultRes
import com.fireworksdk.bridge.models.weakProperty
import com.fireworksdk.bridge.utils.FWCommonUtil
import com.fireworksdk.bridge.utils.FWGlobalDataUtil
import com.fireworksdk.bridge.utils.FWLogUtils
import com.fireworksdk.bridge.utils.FWModelUtils
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.common.MethodChannel.Result
import org.json.JSONArray
import org.json.JSONObject


class FWShoppingModuleHandler(
  private val context: Context,
  channel: MethodChannel?
) : MethodChannel.MethodCallHandler {

  private val methodChannel by weakProperty<MethodChannel?>(channel)
  private val ctaClickChannelResultSet = mutableSetOf<String>()

  override fun onMethodCall(call: MethodCall, result: Result) {
    when (call.method) {
      FWShoppingModuleCallMethod.Init.rawValue -> {
        cartListener()
        productListener()
        result.success(null)
      }

      FWShoppingModuleCallMethod.SetCartIconVisible.rawValue -> {
        val value: Boolean? = call.arguments as Boolean?
        if (value == true) {
          FireworkSdk.shopping.setShoppingCartBehaviour(Shopping.CartBehaviour.Callback)
        } else {
          FireworkSdk.shopping.setShoppingCartBehaviour(Shopping.CartBehaviour.NoCart)
        }
        result.success(null)
      }

      FWShoppingModuleCallMethod.SetCartItemCount.rawValue -> {
        val value: Int? = call.arguments as Int?
        FireworkSdk.shopping.numberOfItemsInCart = value ?: 0
        result.success(null)
      }

      FWShoppingModuleCallMethod.SetCustomClickLinkButtonEnabled.rawValue -> {
        val value: Boolean? = call.arguments as Boolean?
        FWGlobalDataUtil.customClickLinkButtonEnabled = value?:false
        result.success(true)
      }

      FWShoppingModuleCallMethod.SetProductInfoViewConfiguration.rawValue -> {
        val configMap = call.arguments
        val jsonObject = JSONObject(configMap as HashMap<*, *>)
        val configModel = FWProductInfoViewConfigurationDeserializer.deserialize(jsonObject)
        FWLogUtils.d { "FWVideoShoppingModule setProductInfoViewConfiguration: $configModel" }
        if (configModel != null) {
          val isLinkButtonHidden = configModel.linkButton?.isHidden == true
          var text = ShoppingCtaButtonOptions.Text.ADD_TO_CART
          when (configModel.ctaButton?.text) {
            FWCtaButtonTextValue.ShopNow -> {
              text = ShoppingCtaButtonOptions.Text.SHOP_NOW
            }
            else -> {}
          }

          var productCtaButtonText = ProductCardsOptions.Text.SHOP_NOW
          when (configModel.productCard?.ctaButtonText) {
            FWProductCardCtaButtonTextValue.BuyNow -> {
              productCtaButtonText = ProductCardsOptions.Text.BUY_NOW
            }
            else -> {}
          }

          var productCardTheme = ShoppingTheme.DARK
          when (configModel.productCard?.theme) {
            FWProductCardTheme.Light -> {
              productCardTheme = ShoppingTheme.LIGHT
            }
            else -> {}
          }

          var productCardCornerRadius = FWCommonUtil.dpToPx(4.toFloat(), context)
          if (configModel.productCard?.cornerRadius != null) {
            productCardCornerRadius = FWCommonUtil.dpToPx(configModel.productCard.cornerRadius.toFloat(), context)
          }

          var isPriceVisible = true
          if (configModel.productCard?.priceConfiguration?.isHidden != null) {
            isPriceVisible = !configModel.productCard.priceConfiguration.isHidden
          }

          var isCtaVisible = true
          if (configModel.productCard?.isCtaButtonHidden != null) {
            isCtaVisible = !configModel.productCard.isCtaButtonHidden
          }

          FireworkSdk.shopping.setShoppingViewOptions(
            ShoppingViewOptions(
              productCardsOptions = ProductCardsOptions.Default(
                ctaButtonText = productCtaButtonText,
                cornerRadius = productCardCornerRadius.toFloat(),
                isPriceVisible = isPriceVisible,
                isCtaVisible = isCtaVisible,
              ),
              productDetailsOptions = ProductDetailsOptions(
                linkButtonOptions = LinkButtonOptions(isVisible = !isLinkButtonHidden),
                shoppingCtaButtonOptions = ShoppingCtaButtonOptions(text = text),
              ),
              theme = productCardTheme
            ),
          )
        }

        result.success(true)
      }

      FWShoppingModuleCallMethod.SetCustomTapProductCardEnabled.rawValue -> {
        val value: Boolean? = call.arguments as Boolean?
        if (value == true) {
          FireworkSdk.shopping.setOnProductCardClickListener { productId, unitId, productWebUrl, videoInfo ->
            FWLogUtils.d { "FWVideoShoppingModule productCardClick $productId $unitId $productWebUrl" }

            val videoPlaybackDetails = FWModelUtils.convertVideoInfo2PlaybackDetails(videoInfo)

            FWEventUtils.sendProductCardClickEvent(
              methodChannel,
              productWebUrl,
              productId,
              unitId,
              videoPlaybackDetails
            )
          }
        } else {
          FireworkSdk.shopping.setOnProductCardClickListener(null)
        }
        result.success(true)
      }

      FWShoppingModuleCallMethod.ShowShoppingCTALoader.rawValue -> {
        val configMap = call.arguments as HashMap<*, *>
        val callbackId = configMap["callbackId"] as String?
        if (callbackId.isNullOrBlank()) {
          result.success(false)
          return
        }
        if (!ctaClickChannelResultSet.contains(callbackId)) {
          result.success(false)
          return
        }

        FireworkSdk.shopping.setCtaButtonStatus(Shopping.CtaButtonStatus.Loading)
        result.success(true)
      }

      FWShoppingModuleCallMethod.CompleteShoppingCTAHandlingProcess.rawValue -> {
        val configMap = call.arguments as HashMap<*, *>
        val callbackId = configMap["callbackId"] as String?
        if (callbackId.isNullOrBlank()) {
          result.success(false)
          return
        }

        if (!ctaClickChannelResultSet.contains(callbackId)) {
          result.success(false)
          return
        }
        ctaClickChannelResultSet.remove(callbackId)

        val resultMap = configMap["result"] as HashMap<*, *>
        val jsonObject = JSONObject(resultMap)
        val resultModel = FWShoppingCtaResultDeserializer.deserialize(jsonObject)

        val res = resultModel?.res
        val tips = resultModel?.tips
        res?.let {
          when (it) {
            FWShoppingCtaResultRes.Success -> {
              val numberOfItemsInCart = FireworkSdk.shopping.numberOfItemsInCart
              FireworkSdk.shopping.setCtaButtonStatus(Shopping.CtaButtonStatus.Success)
              FireworkSdk.shopping.numberOfItemsInCart = numberOfItemsInCart
            }

            FWShoppingCtaResultRes.Fail -> {
              FireworkSdk.shopping.setCtaButtonStatus(Shopping.CtaButtonStatus.Error)
            }

            FWShoppingCtaResultRes.Loading -> {
              FireworkSdk.shopping.setCtaButtonStatus(Shopping.CtaButtonStatus.Loading)
            }

            else -> {
            }
          }
        }

        if (!tips.isNullOrBlank()) {
          Toast.makeText(context, tips, Toast.LENGTH_LONG).show()
        }

        result.success(true)
      }

      else -> {
        result.notImplemented()
      }
    }
  }

  private fun cartListener() {
    FireworkSdk.shopping.setOnCartClickListener { videoInfo ->
      FWLogUtils.d { "FWVideoShoppingModule cartClicked" }

      val videoPlaybackDetails = FWModelUtils.convertVideoInfo2PlaybackDetails(videoInfo)

      FWEventUtils.sendClickCartIconEvent(methodChannel, videoPlaybackDetails)
    }

    FireworkSdk.shopping.setOnCtaButtonClicked { productId, unitId, productWebUrl, videoInfo ->
      FWLogUtils.d { "FWVideoShoppingModule updateCart, productId: $productId, unitId: $unitId" }
      val channelResult = object : Result {
        override fun success(result: Any?) {
        }

        override fun error(
          errorCode: String,
          errorMessage: String?,
          errorDetails: Any?
        ) {
        }

        override fun notImplemented() {
        }
      }

      val videoPlaybackDetails = FWModelUtils.convertVideoInfo2PlaybackDetails(videoInfo)

      val callbackId = channelResult.hashCode().toString()
      ctaClickChannelResultSet.add(callbackId)
      FWEventUtils.sendShoppingCtaButtonClickEvent(methodChannel, productId, unitId, productWebUrl, videoPlaybackDetails, callbackId, channelResult)
    }

    FireworkSdk.shopping.setOnShoppingErrorListener(
      object : Shopping.OnShoppingErrorListener {
        override fun onShoppingError(error: ShoppingError) {
          when (error) {
            is ShoppingError.CtaButtonClickError.Timeout -> {
            }
            else -> {}
          }
        }
      },
    )
  }

  private fun productListener() {
    FireworkSdk.shopping.setOnProductHydrationListener(object : Shopping.OnProductHydrationListener {
      override fun onProductHydration(products: List<Product>, hydrator: ProductHydrator, videoInfo: VideoInfo) {
        FWLogUtils.d { "FWVideoShoppingModule hydrateProducts, hydrator: ${hydrator.hashCode()}" }
        if (products.isEmpty()) {
          return
        }
        val videoPlaybackDetails = FWModelUtils.convertVideoInfo2PlaybackDetails(videoInfo)
        FWEventUtils.sendUpdateProductsDetailsEvent(methodChannel, products, videoPlaybackDetails, object : Result {
          override fun success(result: Any?) {
            FWLogUtils.d { "FWVideoShoppingModule updateVideoProduct: $result" }
            if (result is ArrayList<*>) {
              val jsonArray = JSONArray(result)
              val videoShoppingProducts =
                FWVideoShoppingProductDeserializer.deserializeArray(jsonArray)
              if (videoShoppingProducts.isNullOrEmpty()) {
                return
              }
              videoShoppingProducts.forEachIndexed { i, vp ->
                val p = products.find { p ->
                  p.id == vp.productId
                }
                p?.let {
                  hydrateProduct(it, vp, hydrator, i)
                }
              }
              hydrator.completeHydration()
            }
          }

          override fun error(
            errorCode: String,
            errorMessage: String?,
            errorDetails: Any?
          ) {
          }

          override fun notImplemented() {
          }
        })

      }
    })

    FireworkSdk.shopping.setOnProductLinkClickListener { productId, unitId, productWebUrl, videoInfo ->
      if (FWGlobalDataUtil.customClickLinkButtonEnabled) {

        val videoPlaybackDetails = FWModelUtils.convertVideoInfo2PlaybackDetails(videoInfo)

        FWEventUtils.sendCustomLinkButtonClickEvent(
          methodChannel,
          productWebUrl,
          productId,
          unitId,
          videoPlaybackDetails
        )
      }
      FWGlobalDataUtil.customClickLinkButtonEnabled
    }
  }

  private fun hydrateProduct(
    product: Product,
    videoProduct: FWVideoShoppingProduct,
    hydrator: ProductHydrator,
    position: Int
  ) {
    FWLogUtils.d { "FWVideoShoppingModule hydrateProduct: $position" }
    product.id?.let { id ->
      val name = videoProduct.name ?: product.name
      val description = (videoProduct.description ?: product.description) ?: ""

      hydrator.hydrate(id) {
        name(name)
        description(description)
        product.units.forEachIndexed { _, unit ->
          unit.id?.let { variantId ->
            val vpUnit = videoProduct.units?.find { vpu ->
              vpu.unitId == variantId
            }
            vpUnit?.let { vpu ->
              val currency = if (vpu.price?.currencyCode != null && vpu.price.currencyCode.isNotBlank())
                CurrencyCode.valueOf(vpu.price.currencyCode)
              else unit.price.currencyCode

              variant(variantId) {
                imageUrl((vpu.imageUrl ?: unit.image?.url) ?: "")
                url(vpu.url ?: unit.url)
                currency(currency)
                price(price = vpu.price?.amount ?: unit.price.amount)
                name((vpu.name ?: unit.name) ?: "")
                isAvailable(vpu.isAvailable) // i != 0
              }
            }
          }
        }
        isAvailable(videoProduct.isAvailable) // position != 0
      }
    }
  }

}
